/*
  The code reads values from a Hall sensor connected to pin A0 on an Arduino Uno board. 
  It continuously reads the analog signal generated by the sensor and prints the raw 
  data to the serial monitor, then determines the polarity of the magnetic field 
  (whether it is south pole or north pole).

  Board: Arduino Uno R3 (or R4)
  Component: Hall Sensor Module
*/

const int hallSensorPin = A0;  // Pin A0 connected to the Hall sensor output

void setup() {
  Serial.begin(9600);             // Initialize serial communication at 9600 bps
  pinMode(hallSensorPin, INPUT);  // Set hall sensor pin as input
}

void loop() {
  int sensorValue = analogRead(hallSensorPin);  // Read analog value from Hall sensor
  Serial.print(sensorValue);                    // Output raw sensor value to Serial Monitor
  delay(200);                                   // Delay for 500 milliseconds

  // Determine magnetic pole based on sensor value
  if (sensorValue >= 700) {
    Serial.print(" - South pole detected");  // South pole detected if value >= 700
  } else if (sensorValue <= 300) {
    Serial.print(" - North pole detected");  // North pole detected if value <= 300
  }

  Serial.println();  // New line for next output
}
